from peaq.utils import ExtrinsicBatch


def storage_add_payload(batch, item_type, item):
    batch.compose_call(
        'PeaqStorage',
        'add_item',
        {
            'item_type': item_type,
            'item': item,
        })


def storage_add(substrate, kp_src, item_type, item):
    batch = ExtrinsicBatch(substrate, kp_src)
    storage_add_payload(batch, item_type, item)
    return batch.execute()


def storage_update_payload(batch, item_type, item):
    batch.compose_call(
        'PeaqStorage',
        'update_item',
        {
            'item_type': item_type,
            'item': item,
        })


def storage_update(substrate, kp_src, item_type, item):
    batch = ExtrinsicBatch(substrate, kp_src)
    storage_update_payload(batch, item_type, item)
    return batch.execute()


def storage_rpc_read(substrate, addr, item_type):
    bl_hsh = substrate.get_block_hash(None)
    data = substrate.rpc_request('peaqstorage_readAttribute', [
                                 addr, item_type, bl_hsh])
    return data['result']



from peaq.utils import ExtrinsicBatch


def funds(substrate, kp_sudo, dsts, token_num, new_reserved=0):
    batch = ExtrinsicBatch(substrate, kp_sudo)
    for dst in dsts:
        batch.compose_sudo_call(
            'Balances',
            'force_set_balance',
            {
                'who': dst,
                'new_free': token_num,
                'new_reserved': new_reserved
            })
    return batch.execute()


# [TODO] Change the API, kp_dst to addr
def fund(substrate, kp_sudo, kp_dst, new_free, new_reserved=0):
    batch = ExtrinsicBatch(substrate, kp_sudo)
    batch.compose_sudo_call(
        'Balances',
        'force_set_balance',
        {
            'who': kp_dst.ss58_address,
            'new_free': new_free,
            'new_reserved': new_reserved
        }
    )
    return batch.execute()



from peaq.utils import ExtrinsicBatch


# [TODO] Change API
def transfer_with_tip(substrate, kp_src, kp_dst_addr, token_num, tip, token_base=0):
    if not token_base:
        token_base = 10 ** 3

    batch = ExtrinsicBatch(substrate, kp_src)
    batch.compose_call(
        'Balances',
        'transfer_keep_alive', {
            'dest': kp_dst_addr,
            'value': token_num * token_base
        }
    )
    return batch.execute(False, None, tip * token_base)


# TODO Change API
def transfer(substrate, kp_src, kp_dst_addr, token_num, token_base=0):
    return transfer_with_tip(substrate, kp_src, kp_dst_addr, token_num, 0, token_base)
